require 'middleclass'
require 'util'
require 'globals'
require 'art'

require 'particle'
require 'wall'
require 'mousecircle'

Game = class('Game')

function Game:load()

	self.score = 0
	self.lives = 2

	setGlobals()
	self.newgame = false
	
	self.spawnrate = 9
	
--	love.physics.setMeter(1)
	self.world = love.physics.newWorld(0, 50, false);
	math.randomseed( os.time() )

	self.mousecircle = MouseCircle:new()

	self.wall1 = Wall:new(self.world,false)
	self.wall2 = Wall:new(self.world,true)

	self.particles = {}

	--local testParticle = Particle:new(self.world,math.random(HWIDTH-100,HWIDTH+100),10,-1)
	--table.insert(self.particles,testParticle)
	--local testParticle2 = Particle:new(self.world,math.random(HWIDTH-100,HWIDTH+100),10,-1)
	--table.insert(self.particles,testParticle2)

	self.stars = {}
	for i=1,100 do
		table.insert( self.stars, {math.random(0,WIDTH),math.random(0,HEIGHT)} )
	end 

	self:spawn()

	self.shoottime = love.timer.getTime()
	self.timeuntilshot = 15
	self.shot = false

	self.failmessage = ""
	self.actuallyGameOver = false
end

function Game:update(dt)
	if self.pause then return end
	
	self.spawnrate = self.spawnrate + 0.08*dt
	self.world:update(dt)

	self.mousecircle:update(dt)

	self.wall1:update(dt)
	self.wall2:update(dt)

--[[
	if (math.random(0,1000) < 5) then
		local testParticle = Particle:new(self.world,math.random(HWIDTH-135,HWIDTH+135),106,-1)
		table.insert(self.particles,testParticle)
	end
]]--
	if (math.random(0,10000) <= self.spawnrate) then
		self:spawn()
	end

	for i=1,#self.particles do
		for j=1,#self.particles do
			self.particles[i]:interactWith(self.particles[j])
		end
	end

	for i=1,#self.particles do
		self.particles[i]:update(dt)
	end

	for i=1,#self.stars do
		self.stars[i][1] = self.stars[i][1]-(10*dt)
	end
	if (math.random(0,1000) <= 1000*dt) then
		table.insert( self.stars, {WIDTH+5,math.random(0,HEIGHT)} )
	end

	self.timeuntilshot = self.timeuntilshot - dt
	if self.timeuntilshot < 0 then
		self:shoot()
	end
end

function Game:spawn()
	local randx = math.random(HWIDTH-135,HWIDTH+135)
	local rxe = 0
	local rye = 0
	while (math.abs(rxe) < 10 or math.abs(rye) < 10) do
		rxe = (50*math.random())-25
		rye = (50*math.random())-25
	end

	local proton = Particle:new(self.world,randx,106,1)
	local electron = Particle:new(self.world,randx+rxe,106+rye,-1)

	local f = 100
	local fx = (20*math.random())-10
	local fy = math.random()*math.random()*5
	proton:applyImpulse(f*fx,f*fy)

	table.insert(self.particles,proton)
	table.insert(self.particles,electron)
end

function Game:shoot()
	
	local protons = {}
	local protoncount = 0

	for i=1,#self.particles do
		local x,y = self.particles[i]:getPosition()
		if inBox({x,y},{HWIDTH-52,459,105,60}) then
			if self.particles[i].alive then
				if self.particles[i].q == -1 then
					self:gameOver("Electrons present - Ion cannon insufficiently ionized")
					self.shot = false
					return
				else
					table.insert(protons,i)
					protoncount = protoncount + 1
				end
			end
		elseif self.particles[i].alive and self.particles[i].q ~= -1 then
			protoncount = protoncount + 1
		end
	end
	if #protons == 0 then
		self:gameOver("No ions present - Ion cannon failed")
		self.shot = false
		return
	end

	for p=1,#protons do
		self.particles[protons[p]]:destroy()
	end

	self.score = self.score+1
	self.shoottime = love.timer.getTime()
	self.timeuntilshot = 15
	self.shot = true
	self.lives = 2

	if protoncount <= 1 then
		self:spawn()
	end
end

function Game:gameOver(message)
	self.shoottime = love.timer.getTime()
	self.timeuntilshot = 15
	self.lives = self.lives - 1
	if self.lives < 0 then
		self.actuallyGameOver = true
		self.pause = true
	end
	self.failmessage = message
end

function Game:draw()
	--love.graphics.setCaption(love.timer.getFPS( ))

	-- background
	love.graphics.setColor(26,26,26,255)
	love.graphics.rectangle("fill",0,0,WIDTH,HEIGHT)

	love.graphics.setColor(180,180,180,255)

	for i=1,#self.stars do
		love.graphics.rectangle("fill",self.stars[i][1],self.stars[i][2],2,2)
	end

	
	love.graphics.setColor(93,93,93,255)
	love.graphics.rectangle("fill",0,246,WIDTH,86)

	love.graphics.setColor(37,53,85,255)
	love.graphics.rectangle("fill",-119,73,263,418)
	love.graphics.rectangle("fill",880,73,263,418)

	love.graphics.setColor(48,69,111,255)
	love.graphics.rectangle("fill",1012,291,90,182)
	love.graphics.rectangle("fill",1012,91,90,182)
	love.graphics.rectangle("fill",902,291,90,182)
	love.graphics.rectangle("fill",902,91,90,182)
	love.graphics.rectangle("fill",30,291,90,182)
	love.graphics.rectangle("fill",30,91,90,182)
	love.graphics.rectangle("fill",-79,291,90,182)
	love.graphics.rectangle("fill",-79,91,90,182)

	-- game
	self.wall1:draw()
	self.wall2:draw()

	for i=1,#self.particles do
		self.particles[i]:draw()
	end

	self.mousecircle:draw()
	
--	self.ball:draw()
	
	--love.graphics.setColor(255,255,255,255)
	--love.graphics.rectangle("fill", HWORLDWIDTH-2.5,0,5,WORLDHEIGHT)
	


	local timesince = love.timer.getTime() - self.shoottime

	
	if self.shot then
		local beamalpha = (2-timesince)/2
		if beamalpha < 0 then beamalpha = 0 end
		love.graphics.setColor(255,0,0,255*beamalpha*beamalpha)
		love.graphics.rectangle("fill", HWIDTH-58,460,58*2,HHEIGHT)
	else
		local beamalpha = (4-timesince)/4
		if beamalpha < 0 then beamalpha = 0 end
		love.graphics.setColor(255,255,255,255*beamalpha*beamalpha)
		love.graphics.setFont(font18)
		love.graphics.printf(self.failmessage,0,HEIGHT-40,WIDTH,"center")
	end

--	if self.timeuntilshot <= 5 then
	love.graphics.setFont(font28)
	love.graphics.setColor(255,255,255,255)
	love.graphics.print(math.ceil(self.timeuntilshot),HWIDTH+68,470)
--	end

	love.graphics.print(self.score,30,30)
	love.graphics.print(self.lives,WIDTH-50,30)

	if self.actuallyGameOver then
		love.graphics.setFont(font42)
		love.graphics.setColor(0,0,0,255)
		love.graphics.printf("GAME OVER",0,HHEIGHT-150,WIDTH,"center")
		love.graphics.setFont(font28)
		love.graphics.printf("Press Esc to restart.",0,HHEIGHT-100,WIDTH,"center")
	end

end

function Game:newGame()
	return self.newgame
end

function Game:keypressed(key)
end


function Game:mousereleased(x,y)
	for i=1,#self.particles do
		self.particles[i]:repelFrom(x,y,self.mousecircle.power)
	end
end
