require 'middleclass'
--require 'art'
require 'util'

Particle = class('Particle')

function Particle:initialize(world,x,y,q)
	self.type = "Particle"
	self.world = world
	
	self.x = x
	self.y = y
	self.q = q

	if q == -1 then -- electron
		self.r = 12
		self.m = 0.1
		self.colour = {11,137,214,255}
	else -- proton
		self.r = 16
		self.m = 1
		self.colour = {244,10,10,255}
	end
	
	self.block = {}
	self.block.body = love.physics.newBody(self.world, self.x, self.y, "dynamic")
	self.block.shape = love.physics.newCircleShape(self.r)
	self.block.fixture = love.physics.newFixture(self.block.body, self.block.shape)
	
	self.block.fixture:setRestitution(0.4)
	self.block.fixture:setFriction(0.0)
	
	self.block.body:setMass(self.m)

	self.alive = true
	
--	self.block.body:setLinearVelocity(vx,vy)
end

function Particle:update(dt)
	self.x, self.y = self.block.body:getPosition()
	if self.x < 0 or self.x > WIDTH then alive = false return end
	if self.y < 0 or self.y > HEIGHT then alive = false return end
	
	local mu = 0.4 * self.m
	local vx,vy = self.block.body:getLinearVelocity()
	self.block.body:applyForce(-vx*mu,-vy*mu)
end

function Particle:draw()
	if not self.alive then return end

	love.graphics.setColor(self.colour)
	love.graphics.setFont(vanillafont18)
	
	self.x, self.y = self.block.body:getPosition()
	love.graphics.circle("fill",self.x,self.y,self.r-3,16)
	
	if (self.q == -1) then
		love.graphics.print("e-",self.x+self.r-5,self.y+self.r-5)
	else
		love.graphics.print("H+",self.x+self.r-5,self.y+self.r-5)
	end
end

function Particle:repelFrom(x,y,power)

	if not self.alive then return end

	local qq = -200000*power*self.m

	local sx,sy = self:getPosition()
	local ox,oy = x,y

	if sx == ox and sy == oy then return end

	local dist = distance({sx,sy},{ox,oy})

	if dist < self.r then dist = self.r end
	--local force = (qq)/(dist*dist)
	local force = (qq)/(dist)

	local dx1 = (ox-sx)/dist
	local dy1 = (oy-sy)/dist

	self:applyImpulse(dx1*force,dy1*force)
end

function Particle:applyMassImpulse(x,y)
	self:applyImpulse(self.m*x,self.m*y)
end

function Particle:applyImpulse(x,y)
	if not self.alive then return end
	self.block.body:applyForce(x,y)
end

function Particle:getPosition()
	return self.block.body:getPosition()
end

function Particle:interactWith(other)
	if not self.alive then return end
	if not other.alive then return end

	local qq = 65000

	local sx,sy = self:getPosition()
	local ox,oy = other:getPosition()

	if sx == ox and sy == oy then return end

	local dist = distance({sx,sy},{ox,oy})
	local force = (qq)/(dist*dist)

	local dx1 = (ox-sx)/dist
	local dy1 = (oy-sy)/dist

	if (self.q == other.q) then force = -force end

	self.block.body:applyForce(dx1*force,dy1*force)
	--self.block.body:applyForce(fx,fy)
end

function Particle:destroy()
	self.alive = false
	self.block.body:setActive(false)
end
